@@ -202,6 +202,13 @@ class TestQuery(WebserviceTest): # pragma: no cover
         self.assertRaises(TypeError,  self.q.add_sort_order, "Employee.name", "up")
         self.assertRaises(QueryError, self.q.add_sort_order, "Employee.id", "desc")
 
+    def testUCSortOrder(self):
+        """Sort-Order directions should be accepted in upper case"""
+        self.q.add_view("Employee.name", "Employee.age", "Employee.fullTime")
+        self.q.add_sort_order("Employee.age", "ASC")
+        self.assertEqual(str(self.q.get_sort_order()), "Employee.age asc")
+        self.q.add_sort_order("Employee.fullTime", "DESC")
+        self.assertEqual(str(self.q.get_sort_order()), "Employee.age asc,Employee.fullTime desc")
 
     def testConstraintPathProblems(self):
         """Queries should not add constraints with bad paths to themselves"""
@@ -666,14 +673,57 @@ class TestQueryResults(WebserviceTest): # pragma: no cover
             q_res = self.query.all("rr")
             t_res = self.template.all("rr")
             for results in [q_res, t_res]:
-                assertEqual(results[0]["age"], 'bar')
-                assertEqual(results[1]["Employee.age"], 1.23)
-                assertEqual(results[2][0], True)
+                assertEqual(results[0]["age"], 'bar') # Can index by short path
+                assertEqual(results[1]["Employee.age"], 1.23) # or by full path
+                assertEqual(results[2][0], True) # or by numerical index
                 assertEqual(len(results), 3)
                 for row in results:
                     assertEqual(len(row), 3)
         self.do_unpredictable_test(logic)
 
+    def testResultRowIterability(self):
+        """Result rows should iterate as lists"""
+        def logic():
+            q_res = self.query.all("rr")
+            t_res = self.template.all("rr")
+            for results in [q_res, t_res]:
+                # 'in' as test of iterability
+                self.assertTrue("bar" in results[0])
+                self.assertTrue(1.23 in results[1])
+                self.assertTrue(True in results[2])
+                # Should be able to actually iterate
+                count = 0
+                for val in results[0]:
+                    count += 1
+                    self.assertTrue(0 < count < 4)
+
+        self.do_unpredictable_test(logic)
+
+    def testResultRowDictBehaviour(self):
+        """Result rows should allow iteration using items() and iterkeys()"""
+        def logic():
+            q_res = self.query.all("rr")
+            t_res = self.template.all("rr")
+            for results in [q_res, t_res]:
+                r = results[0]
+                count = 0
+                for (k, v) in r.items():
+                    count += 1
+                    self.assertTrue(0 < count < 4)
+                count = 0
+                self.assertEqual([("Employee.name", 'foo'), ("Employee.age", 'bar'), ("Employee.id", 'baz')], r.items())
+                for (k, v) in r.iteritems():
+                    count += 1
+                    self.assertTrue(0 < count < 4)
+                self.assertEqual([pair for pair in r.iteritems()], r.items())
+                self.assertEqual(r.keys(), self.query.views)
+                self.assertEqual(r.values(), r.to_l())
+                self.assertEqual(zip(r.values(), r.keys()), zip(r.itervalues(), r.iterkeys()))
+                self.assertTrue(r.has_key("age"))
+                self.assertTrue(r.has_key("Employee.age"))
+                self.assertTrue(not r.has_key("Employee.foo"))
+
+        self.do_unpredictable_test(logic)
 
     def testResultsDict(self):
         """Should be able to get results as one dictionary per row"""
